package data.hullmods;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;

public class NeutrinoOmegaUpgrade extends BaseHullMod {

    //private static final List allowedIds = new ArrayList();

        private static final Set<String> BLOCKED_HULLMODS = new HashSet<>();
        static
        {
            // These hullmods will automatically be removed
            // Not as elegant as blocking them in the first place, but
            // this method doesn't require editing every hullmod's script
            BLOCKED_HULLMODS.add("targetingunit");
            BLOCKED_HULLMODS.add("dedicated_targeting_core");
 
        }
    
	private static Map mag1 = new HashMap<>();
	static {
		mag1.put(HullSize.FIGHTER, 0f);
		mag1.put(HullSize.FRIGATE, 20f);
		mag1.put(HullSize.DESTROYER, 20f);
		mag1.put(HullSize.CRUISER, 20f);
		mag1.put(HullSize.CAPITAL_SHIP, 20f);
	}

	private static Map mag2 = new HashMap();
	static {
		mag2.put(HullSize.FRIGATE, 75f);
		mag2.put(HullSize.DESTROYER, 125f);
		mag2.put(HullSize.CRUISER, 175f);
		mag2.put(HullSize.CAPITAL_SHIP, 225f);
	}	
	
	public static final float HULL_BONUS = -25f;
	public static final float SHIELD_UPKEEP_BONUS = 50f;
	
        @Override
	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {

		//weapon aim
		stats.getAutofireAimAccuracy().modifyFlat(id, 60f);	

		//weapon range
		stats.getBallisticWeaponRangeBonus().modifyPercent(id, (Float) mag1.get(hullSize));
		stats.getEnergyWeaponRangeBonus().modifyPercent(id, (Float) mag1.get(hullSize));
		
		//armor + hull
		stats.getArmorBonus().modifyFlat(id, (Float) mag2.get(hullSize));
		stats.getHullBonus().modifyPercent(id, HULL_BONUS);
		
		//increase shield upkeep
		stats.getShieldUpkeepMult().modifyMult(id, 1f + SHIELD_UPKEEP_BONUS * 0.01f);		
		}
	
        @Override
	public String getDescriptionParam(int index, HullSize hullSize) {

		if (index == 0) return "" + ((Float) mag1.get(HullSize.FRIGATE)).intValue();
		if (index == 1) return "" + ((Float) mag1.get(HullSize.DESTROYER)).intValue();
		if (index == 2) return "" + ((Float) mag1.get(HullSize.CRUISER)).intValue();
		if (index == 3) return "" + ((Float) mag1.get(HullSize.CAPITAL_SHIP)).intValue();			
		if (index == 4) return "" + ((Float) mag2.get(HullSize.FRIGATE)).intValue();
		if (index == 5) return "" + ((Float) mag2.get(HullSize.DESTROYER)).intValue();
		if (index == 6) return "" + ((Float) mag2.get(HullSize.CRUISER)).intValue();
		if (index == 7) return "" + ((Float) mag2.get(HullSize.CAPITAL_SHIP)).intValue();
		if (index == 8) return "" + (int) HULL_BONUS;
		if (index == 9) return "" + (int) SHIELD_UPKEEP_BONUS;		
		
		return null;
	}
        
    @Override
    public void applyEffectsAfterShipCreation(ShipAPI ship, String id)
    {
        for (String tmp : BLOCKED_HULLMODS)
        {
            if (ship.getVariant().getHullMods().contains(tmp))
            {
                ship.getVariant().removeMod(tmp);
            }
        }
    }        
    
    @Override
    public boolean isApplicableToShip(ShipAPI ship) {
        // Allows any ship with a Neutrino hull id  
        return ship.getHullSpec().getHullId().startsWith("neutrino_");
    }        
    /*
    @Override
    public boolean isApplicableToShip(ShipAPI ship)
    {
        // Allows any ship with a neutrino hull id
        return ( ( ship.getHullSpec().getHullId().startsWith("neutrino_") && !ship.getVariant().getHullMods().contains("dedicated_targeting_core")) ||
				( ship.getHullSpec().getHullId().startsWith("neutrino_") && !ship.getVariant().getHullMods().contains("targetingunit")) );

        
        
    }
    */
}
