package data.hullmods;

import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.RepairTrackerAPI;
import com.fs.starfarer.api.fleet.RepairTrackerAPI.CREvent;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class NeutrinoSigmaUpgrade extends BaseHullMod {

    private static final Set<String> BLOCKED_HULLMODS = new HashSet<>();

    static {
        // These hullmods will automatically be removed
        // Not as elegant as blocking them in the first place, but
        // this method doesn't require editing every hullmod's script
        BLOCKED_HULLMODS.add("extendedshieldemitter");

    }

    //private static final List allowedIds = new ArrayList();
    public static final float SHIELD_ARC_BONUS = 30f;
    public static final float GUIDANCE_IMPROVEMENT = 0.5f;
    public static final float HEALTH_BONUS = 25f;
    public static final float SHIELD_BONUS_UNFOLD = 125f;
    public static final float VENT_RATE_BONUS = -25f;

    @Override
    public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
        //shield arc
        stats.getShieldArcBonus().modifyFlat(id, SHIELD_ARC_BONUS);

        //missile maneuverability
        stats.getMissileGuidance().modifyFlat(id, GUIDANCE_IMPROVEMENT);

        stats.getMissileMaxSpeedBonus().modifyPercent(id, 5f);
        stats.getMissileAccelerationBonus().modifyPercent(id, 25f);
        stats.getMissileMaxTurnRateBonus().modifyPercent(id, 10f);
        stats.getMissileTurnAccelerationBonus().modifyPercent(id, 25f);

        //weapon aim
        stats.getAutofireAimAccuracy().modifyFlat(id, 40f);

        //weapon hp
        stats.getWeaponHealthBonus().modifyPercent(id, HEALTH_BONUS);

        //shield expansion rate
        stats.getShieldUnfoldRateMult().modifyPercent(id, SHIELD_BONUS_UNFOLD);

        //flux vent rate
        stats.getVentRateMult().modifyPercent(id, VENT_RATE_BONUS);

    }
//
//    @Override
//    public void advanceInCampaign(FleetMemberAPI member, float amount) {
//        List<CREvent> CREvents = member.getRepairTracker().getRecentEvents();
//        if (!CREvents.isEmpty()) {
//            for (CREvent CREvent : CREvents) {
//                RepairTrackerAPI RT = member.getRepairTracker();
//                if (CREvent.getText().equals("Hull damage sustained")) {
//                    CREvent.text = "Hull damage sustained (partially restored by Neutrino Sigma Upgrade)";
//                    CREvent.crAmount /= 2;
//                    RT.setCR(Math.min(RT.getMaxCR(), RT.getCR() - CREvent.crAmount));
//                }
//            }
//        }
//    }

    @Override
    public String getDescriptionParam(int index, HullSize hullSize) {
        if (index == 0) {
            return "" + (int) SHIELD_ARC_BONUS;
        }
        if (index == 1) {
            return "" + (int) (GUIDANCE_IMPROVEMENT * 50f);
        }
        if (index == 2) {
            return "" + (int) HEALTH_BONUS;
        }
        if (index == 3) {
            return "" + (int) SHIELD_BONUS_UNFOLD;
        }
        if (index == 4) {
            return "" + (int) VENT_RATE_BONUS;
        }
        return null;
    }

    @Override
    public void applyEffectsAfterShipCreation(ShipAPI ship, String id) {
        for (String tmp : BLOCKED_HULLMODS) {
            if (ship.getVariant().getHullMods().contains(tmp)) {
                ship.getVariant().removeMod(tmp);
            }
        }
    }

    @Override
    public boolean isApplicableToShip(ShipAPI ship) {
        // Allows any ship with a Neutrino hull id  
        return ship.getHullSpec().getHullId().startsWith("neutrino_");
    }
    /*
     @Override
     public boolean isApplicableToShip(ShipAPI ship)
     {
     // Allows any ship with a neutrino hull id
     return ( ( ship.getHullSpec().getHullId().startsWith("neutrino_") && !ship.getVariant().getHullMods().contains("dedicated_targeting_core")) ||
     ( ship.getHullSpec().getHullId().startsWith("neutrino_") && !ship.getVariant().getHullMods().contains("targetingunit")) );

        
        
     }
     */
}
