package data.scripts.plugins;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BaseEveryFrameCombatPlugin;
import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamagingProjectileAPI;
import com.fs.starfarer.api.input.InputEventAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.util.Neutrino_ParticlesEffectLib;

import java.util.List;
import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import org.lwjgl.util.vector.Vector2f;

import org.lazywizard.lazylib.combat.entities.AnchoredEntity;
import org.lazywizard.lazylib.MathUtils;

import org.dark.shaders.distortion.DistortionShader;
import org.dark.shaders.distortion.WaveDistortion;
import org.dark.shaders.light.LightShader;
import org.dark.shaders.light.StandardLight;

public class Neutrino_ProjectilesEffect extends BaseEveryFrameCombatPlugin {

    private CombatEngineAPI engine;

    // DATA
    // For phaseMissileEffect
    private static final float phaseFadeTime = 2f;
    private final IntervalUtil flashInterval = new IntervalUtil(0.01F, 0.01F);
    ////

    //Projectile List
    private static final String neutrino_phase_missile_ID = "neutrino_phase_missile";
    private static final String neutrino_phase_missile_payload_ID = "neutrino_phase_missile2";
    private static final String neutrino_sapper_missile_ID = "neutrino_sapper_missile";
    private static final String neutrino_heavypulsar_shot_ID = "neutrino_heavypulsar_shot";
    private static final String neutrino_pulsar_shot_ID = "neutrino_pulsar_shot";

    public static Map<BeamAPI, Vector2f> beamsNeedCompensate = new HashMap<>();

    @Override
    public void advance(float amount, List<InputEventAPI> events) {
        if (engine == null || engine.isPaused()) {
            return;
        }
        List<DamagingProjectileAPI> projectiles = engine.getProjectiles();
        if (!projectiles.isEmpty()) {
            for (DamagingProjectileAPI proj : projectiles) {
                String projSpecId = proj.getProjectileSpecId();
                if (projSpecId != null) {
                    switch (projSpecId) {
                        case neutrino_phase_missile_ID:
                        case neutrino_phase_missile_payload_ID:
                            phaseMissileEffect(proj, amount);
                            break;
                        case neutrino_sapper_missile_ID:
                            Vector2f vel = proj.getVelocity();
                            vel.normalise(vel);
                            float mod = Math.max(150 + proj.getSource().getMutableStats().getMissileMaxSpeedBonus().computeEffective(800f)
                                    - (proj.getSource().getMutableStats().getMissileMaxSpeedBonus().computeEffective(400f) * proj.getElapsed()),
                                    150f);
                            vel.scale(mod);
                            break;
                        case neutrino_heavypulsar_shot_ID:
                        case neutrino_pulsar_shot_ID:
                            if (proj.didDamage()) {
                                return;
                            }
                            float speed = proj.getVelocity().length();
                            Vector2f tmp = MathUtils.getPointOnCircumference(proj.getLocation(), speed * amount, MathUtils.clampAngle(proj.getFacing() + 180));
                            for (int i = 0; i < 4; i++) {
                                Vector2f random = MathUtils.getRandomPointOnLine(tmp, proj.getLocation());
                                Neutrino_ParticlesEffectLib.AddParticles(
                                        1, 2, true,
                                        random, 5,
                                        proj.getFacing(), 30,
                                        0, speed * 0.03f,
                                        7, 11,
                                        0.9f, 1f,
                                        0.3f, 0.5f,
                                        Color.WHITE);
                            }
                            break;
                        default:
                    }
                }
            }
        }
    }

    public void phaseMissileEffect(DamagingProjectileAPI proj, float amount) {
        float elapsed = proj.getElapsed();
        Vector2f loc = proj.getLocation();
        MissileAPI missile = (MissileAPI) proj;
        switch (proj.getProjectileSpecId()) {
            case neutrino_phase_missile_ID:
                missile.getSpriteAPI().setAlphaMult(Math.min(elapsed * 0.01f, 0.1f));
                float maxFlightTime = missile.getMaxFlightTime();
                if (elapsed < amount) {
                    float dis = proj.getSource().getCollisionRadius() + MathUtils.getRandomNumberInRange(20f, 50f);
                    Vector2f mov = MathUtils.getRandomPointOnCircumference(null, dis);
                    Vector2f.add(loc, mov, loc);
                    WaveDistortion wave = new WaveDistortion();
                    wave.setLocation(loc);
                    wave.setIntensity(25f);
                    wave.setLifetime(0.5f);
                    wave.setSize(15f);
                    wave.fadeOutIntensity(0.25f);
                    DistortionShader.addDistortion(wave);
                    StandardLight light = new StandardLight();
                    light.setLocation(loc);
                    light.setColor(new Color(255, 175, 255, 50));
                    light.setSize(10f);
                    light.setIntensity(1f);
                    light.fadeOut(0.25f);
                    LightShader.addLight(light);
                }
                if (maxFlightTime - elapsed < phaseFadeTime) {
                    missile.getSpriteAPI().setAlphaMult(Math.min((Math.max(maxFlightTime - elapsed, 0f)) / phaseFadeTime * 0.1f, MathUtils.getRandomNumberInRange(0f, 0.1f)));
                }
                break;
            case neutrino_phase_missile_payload_ID:
                if (elapsed < amount) {
                    Vector2f zero = new Vector2f();
                    WaveDistortion wave = new WaveDistortion();
                    wave.setLocation(loc);
                    wave.setIntensity(50f);
                    wave.setLifetime(1f);
                    wave.setSize(50f);
                    wave.fadeOutIntensity(0.5f);
                    DistortionShader.addDistortion(wave);
                    StandardLight light = new StandardLight();
                    light.attachTo(proj);
                    light.setLocation(loc);
                    light.setColor(new Color(255, 175, 255, 50));
                    light.setSize(30f);
                    light.setIntensity(0.2f);
                    light.fadeOut(0.3f);
                    LightShader.addLight(light);
                    Global.getSoundPlayer().playSound("system_phase_cloak_collision", 1f, 1f, loc, zero);
                    Global.getSoundPlayer().playSound("neutrino_lightlaunch", 0.5f, 0.5f, loc, zero);
                }
                if (elapsed <= 0.4f) {
                    flashInterval.advance(amount);
                    missile.getSpriteAPI().setAlphaMult(elapsed * 2f + 0.2f);
                    if (flashInterval.intervalElapsed()) {
                        Vector2f randomPoint = MathUtils.getRandomPointOnCircumference(loc, 5f);
                        CombatEntityAPI anchor1 = new AnchoredEntity(proj, randomPoint);
                        engine.spawnEmpArc(proj.getSource(), loc, anchor1, anchor1, DamageType.ENERGY, 0f, 0f, 50f, null, 2f,
                                new Color(51, 37, 51, 100), new Color(255, 0, 255, 0));
                    }
                }
                break;
        }
    }

    @Override
    public void init(CombatEngineAPI engine) {
        this.engine = engine;
    }

}
