package data.scripts.weapons;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.combat.BeamAPI;
import com.fs.starfarer.api.combat.BeamEffectPlugin;
import com.fs.starfarer.api.combat.CollisionClass;
import com.fs.starfarer.api.combat.CombatAsteroidAPI;
import com.fs.starfarer.api.combat.CombatEngineAPI;
import com.fs.starfarer.api.combat.CombatEntityAPI;
import com.fs.starfarer.api.combat.DamageType;
import com.fs.starfarer.api.combat.MissileAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.graphics.SpriteAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import data.scripts.plugins.Neutrino_FakeBeamPlugin;
import data.scripts.util.Neutrino_CollisionUtilsEX;
import data.scripts.util.Neutrino_ParticlesEffectLib;
import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import org.dark.shaders.distortion.DistortionShader;
import org.dark.shaders.distortion.WaveDistortion;
import org.dark.shaders.light.LightShader;
import org.dark.shaders.light.StandardLight;
import org.lazywizard.lazylib.CollisionUtils;
import org.lazywizard.lazylib.MathUtils;
import org.lazywizard.lazylib.VectorUtils;
import org.lazywizard.lazylib.combat.CombatUtils;
import org.lwjgl.util.vector.Vector2f;

public class NeutSilverLanceEffect implements BeamEffectPlugin {

    public NeutSilverLanceEffect() {
    }
    // for VFX
    private final SpriteAPI beamSprite = Global.getSettings().getSprite("graphics/fx/beamcore.png");
    private final Color beamColor = Color.white;
    private final float beamWidth = 10f;
    private final float duration = 0.1f;
    private final float fadeIn = 0.02f;
    private final float fadeOut = 0.02f;
    private final float particlesDensity = 5f;
    private final float hitSize = 50f;

    // for dmg
    private final float shieldHitHardFulxRatio = 0.6f;
    private final float baseShieldHitDmg = 1000f;
    private final float dmgMultAfterShieldHit = 0.1f;
    private final float baseHullHitDmg = 500f;
    private final float onHitDmgMult = 0.12f;
    private final float subDmg = 100f;
    private final float piercingDmgInterval = 10f;
    private final float EMPRatio = 1f;
    private final float EMPRatioAfterShield = 2f;
//    private final DamageType damageType = DamageType.KINETIC;
    private int times = 5;
    private final IntervalUtil firingInterval = new IntervalUtil(0.08f, 0.08f);

    private final Vector2f zero = new Vector2f(0, 0);
    private final List<PiercedEntity> PiercedEntities = new ArrayList<>();

    @Override
    public void advance(float amount, CombatEngineAPI engine, BeamAPI beam) {
        if (times > 0 && beam.getWeapon().getChargeLevel() >= 1) {
            firingInterval.advance(amount);
            if (firingInterval.intervalElapsed()) {
                final float mult = beam.getSource().getMutableStats().getBeamWeaponDamageMult().getModifiedValue() * beam.getSource().getMutableStats().getEnergyWeaponDamageMult().getModifiedValue();
                final float fullDmg = beam.getWeapon().getDerivedStats().getBurstDamage() * mult;
                float shieldHitDmg = baseShieldHitDmg * mult;
                float hullHitDmg = baseHullHitDmg * mult;
                float dmg = fullDmg;
                ShipAPI source = beam.getSource();
                final float range = beam.getWeapon().getRange();
                final float facing = beam.getWeapon().getCurrAngle();
                final Vector2f beamForm = beam.getWeapon().getLocation();
                Vector2f beamTo = MathUtils.getPointOnCircumference(beamForm, range, facing);
                final Vector2f midPoint = MathUtils.getMidpoint(beamForm, beamTo);
                DamageType damageType = beam.getWeapon().getDamageType();

                // Collect all entitise that may be hit by beam.
                for (CombatEntityAPI e : CombatUtils.getEntitiesWithinRange(midPoint, range / 2f)) {
                    // skip what can't be hit
                    if (e.getCollisionClass() == CollisionClass.NONE) {
                        continue;
                    }
                    // check for ship hit. find out shield hit point, hull hit point, and hull exit point.
                    if (e instanceof ShipAPI) {
                        ShipAPI s = (ShipAPI) e;
                        if (s == source || (s.isFighter() && s.getOwner() == source.getOwner())) {
                            continue;
                        }
                        PiercedEntity piercedEntity = new PiercedEntity();
                        piercedEntity.entity = e;
                        piercedEntity.shieldHit = Neutrino_CollisionUtilsEX.getShieldCollisionPoint(beamForm, beamTo, s, false);
                        piercedEntity.hullHit = CollisionUtils.getCollisionPoint(beamForm, beamTo, e);
                        if (piercedEntity.hullHit == null && piercedEntity.shieldHit == null) {
                            continue;
                        }
                        piercedEntity.exit = CollisionUtils.getCollisionPoint(beamTo, beamForm, e);
                        PiercedEntities.add(piercedEntity);
                    }
                    // check for asteroid hit. find out hit point and exit point.                
                    if (e instanceof CombatAsteroidAPI) {
                        PiercedEntity piercedEntity = new PiercedEntity();
                        piercedEntity.entity = e;
                        piercedEntity.hullHit = Neutrino_CollisionUtilsEX.getCollisionPointOnCircle(beamForm, beamTo, e.getLocation(), e.getCollisionRadius());
                        if (piercedEntity.hullHit == null) {
                            continue;
                        }
                        piercedEntity.exit = Neutrino_CollisionUtilsEX.getCollisionPointOnCircle(beamTo, beamForm, e.getLocation(), e.getCollisionRadius());
                        PiercedEntities.add(piercedEntity);
                    }
                    // check for missile hit, just find out the hit point.
                    if (e instanceof MissileAPI) {
                        PiercedEntity piercedEntity = new PiercedEntity();
                        piercedEntity.entity = e;
                        piercedEntity.hullHit = CollisionUtils.getCollisionPoint(beamForm, beamTo, e);
                        if (piercedEntity.hullHit == null) {
                            continue;
                        }
                        PiercedEntities.add(piercedEntity);
                    }
                }

                Vector2f tmp = new Vector2f(beamForm);
                boolean end = false;

                if (!PiercedEntities.isEmpty()) {
                    // sort all hit point by distance to beamForm.
                    if (PiercedEntities.size() > 1) {
                        Collections.sort(PiercedEntities, new SortByDistance(beamForm));
                    }
                    Vector2f beamDir = Vector2f.sub(beamTo, beamForm, null);
                    // ...and check it one by one.
                    for (PiercedEntity pe : PiercedEntities) {
                        // re-init the dmgOnHit.
                        float dmgOnHit = fullDmg * onHitDmgMult;
                        // hit missile? just apply damage and pass it.
                        if (pe.entity instanceof MissileAPI) {
                            if (Vector2f.dot(beamDir, Vector2f.sub(pe.hullHit, tmp, null)) <= 0) {
                                continue;
                            }
                            dmgOnHit = pe.entity.getMaxHitpoints() < dmgOnHit ? pe.entity.getMaxHitpoints() : dmgOnHit;
                            engine.applyDamage(pe.entity, pe.hullHit, dmgOnHit, damageType, 0f, false, true, source);
                            dmg -= dmgOnHit;
                            continue;
                        }
                        // check for a shieldHit. 
                        boolean hitShield = false;
                        float targetShieldDmgFluxRaido = 1f;
                        if (pe.shieldHit != null) {
                            // hmm, just make sure this hit point is on the right direction.
                            if (Vector2f.dot(beamDir, Vector2f.sub(pe.shieldHit, tmp, null)) <= 0) {
                                continue;
                            }
                            // draw a fake beam!
                            Neutrino_FakeBeamPlugin.renderFakeBeam(tmp, pe.shieldHit, beamWidth, duration, fadeIn, fadeOut, beamSprite, beamColor);
                            // apply a particles path effect!
                            Neutrino_ParticlesEffectLib.AddParticlesOnSegment(
                                    tmp, pe.shieldHit,
                                    particlesDensity, 2, false,
                                    1,
                                    30f,
                                    10, 50,
                                    2, 4,
                                    0.8f, 1f,
                                    0.3f, 0.6f,
                                    Color.WHITE);
                            // draw an on hit effect!
                            engine.addHitParticle(
                                    pe.shieldHit,
                                    zero,
                                    (float) Math.random() * hitSize / 2 + hitSize,
                                    1,
                                    (float) Math.random() * duration / 2 + duration,
                                    Color.WHITE
                            );
                            engine.addHitParticle(
                                    pe.shieldHit,
                                    zero,
                                    (float) Math.random() * hitSize / 4 + hitSize / 2,
                                    1,
                                    0.1f,
                                    Color.WHITE
                            );
                            // move the tmp point for next check
                            tmp = pe.shieldHit;
                            targetShieldDmgFluxRaido = pe.entity.getShield().getFluxPerPointOfDamage();
                            float dmgOnshield = shieldHitDmg * (targetShieldDmgFluxRaido == 0 ? 100f : 1 / pe.entity.getShield().getFluxPerPointOfDamage());
                            // check if we are run out of damage.
                            if (dmg <= dmgOnshield) {
                                dmgOnshield = dmg;
                                beamTo = pe.shieldHit;
                                end = true;
                            }
                            // apply damage on shield!
                            engine.applyDamage(pe.entity, pe.shieldHit, dmgOnshield * shieldHitHardFulxRatio, damageType, 0f, false, false, source);
                            engine.applyDamage(pe.entity, pe.shieldHit, dmgOnshield * (1f - shieldHitHardFulxRatio), damageType, 0f, false, true, source);
                            hitShield = true;
                            dmg -= dmgOnshield;
                            // if we out of damage, break here.
                            if (end == true) {
                                break;
                            }
                        }
                        // check for a hull hit.
                        if (pe.hullHit != null) {
                            // ...just in case as above.
                            if (Vector2f.dot(beamDir, Vector2f.sub(pe.hullHit, tmp, null)) <= 0) {
                                continue;
                            }
                            // VFXs
                            Neutrino_FakeBeamPlugin.renderFakeBeam(tmp, pe.hullHit, beamWidth, duration, fadeIn, fadeOut, beamSprite, beamColor);
                            Neutrino_ParticlesEffectLib.AddParticlesOnSegment(
                                    tmp, pe.hullHit,
                                    particlesDensity, 2, false,
                                    1,
                                    30f,
                                    10, 50,
                                    2, 4,
                                    0.8f, 1f,
                                    0.3f, 0.6f,
                                    Color.WHITE);
                            engine.addHitParticle(
                                    pe.hullHit,
                                    zero,
                                    (float) Math.random() * hitSize / 2 + hitSize,
                                    1,
                                    (float) Math.random() * duration / 2 + duration,
                                    Color.WHITE
                            );
                            engine.addHitParticle(
                                    pe.hullHit,
                                    zero,
                                    (float) Math.random() * hitSize / 4 + hitSize / 2,
                                    1,
                                    0.1f,
                                    Color.WHITE
                            );
                            if (!hitShield) {
                                float dmgOnHull = hullHitDmg;

                                if (dmg <= dmgOnHull) {
                                    dmgOnHull = dmg;
                                    beamTo = pe.hullHit;
                                    end = true;
                                }
                                engine.applyDamage(pe.entity,
                                        pe.hullHit,
                                        dmgOnHull,
                                        damageType,
                                        0f,
                                        true, false, source);
                                dmg -= dmgOnHull;
                                if (end == true) {
                                    break;
                                }
                            }

                            if (Vector2f.dot(pe.exit, beamTo) <= 0) {
                                end = true;
                            }

                            float piercingDmg = subDmg * mult;
                            float piercingEMP = piercingDmg * EMPRatio;
                            if (hitShield) {
                                piercingEMP *= EMPRatioAfterShield;
                            }
                            float piercingSubDmg = piercingDmg;
                            if (hitShield) {
                                piercingSubDmg *= dmgMultAfterShieldHit;
                                piercingEMP *= dmgMultAfterShieldHit * targetShieldDmgFluxRaido;
                            }
                            Vector2f dir = VectorUtils.getDirectionalVector(pe.hullHit, pe.exit);
                            dir.scale(piercingDmgInterval);
                            Vector2f pTmp = new Vector2f(pe.hullHit);
                            while (dmg > 0 && Vector2f.dot(dir, Vector2f.sub(pe.exit, pTmp, null)) > 0) {
                                if (dmg <= piercingDmg) {
                                    piercingDmg = dmg;
                                    beamTo = pTmp;
                                    end = true;
                                }
                                dmg -= piercingDmg;
//                            Vector2f to = MathUtils.getRandomPointInCircle(pTmp, 20f);
//                            while (!CollisionUtils.isPointWithinBounds(to, pe.entity)) {
//                                to = MathUtils.getRandomPointInCircle(pTmp, 20f);
//                            }
//                            AnchoredEntity toE = new AnchoredEntity(pe.entity, to);
//                            engine.spawnEmpArc(source,
//                                    form,
//                                    pe.entity,
//                                    toE,
//                                    damageType,
//                                    0, 0,
//                                    20, null, 0, Color.BLACK, beamColor);
                                engine.applyDamage(pe.entity,
                                        pTmp,
                                        piercingSubDmg,
                                        damageType,
                                        piercingEMP,
                                        true, false, source);
                                Vector2f.add(pTmp, dir, pTmp);
                            }

                            if (end == true) {
                                break;
                            }

                            // seens like fighter can cause trouble in some case, so we should do some trick on it.
                            if (pe.entity instanceof ShipAPI && ((ShipAPI) pe.entity).isFighter()) {

                            } else if (PiercedEntities.iterator().hasNext() && !CollisionUtils.isPointWithinBounds(beamTo, pe.entity)) {
                                // do some beam exit point VFX
                                tmp = pe.exit;
                                engine.applyDamage(pe.entity,
                                        tmp,
                                        piercingSubDmg,
                                        damageType,
                                        piercingEMP,
                                        true, false, source);
                                Neutrino_ParticlesEffectLib.AddParticles(
                                        10, 1, false,
                                        tmp, 5f,
                                        facing, 30f,
                                        30, 200,
                                        3, 5,
                                        0.8f, 1f,
                                        0.5f, 2f,
                                        Color.WHITE);
                            }
                        }
                    }
                }
                if (!end) {
                    Neutrino_FakeBeamPlugin.renderFakeBeam(tmp, beamTo, beamWidth, duration, fadeIn, fadeOut, beamSprite, beamColor);
                    Neutrino_ParticlesEffectLib.AddParticlesOnSegment(
                            tmp, beamTo,
                            particlesDensity, 2, false,
                            1,
                            30f,
                            10, 50,
                            2, 4,
                            0.8f, 1f,
                            0.3f, 0.6f,
                            Color.WHITE);
                }
                PiercedEntities.clear();
                // SFX
                Global.getSoundPlayer().playSound("neutrino_silverlance_fire", 1, 1, beamForm, zero);
                // extra VFX
                WaveDistortion wave = new WaveDistortion();
                wave.setLocation(beamForm);
                wave.setIntensity(5f);
                wave.setLifetime(0.14f);
//                wave.fadeInIntensity(0.1f);
//                wave.setAutoFadeSizeTime(Float.MAX_VALUE);
//                wave.setAutoFadeIntensityTime(0.2f);
                wave.setSize(MathUtils.getDistance(beamForm, beamTo));
                wave.setArc(facing + 180 - 2.5f, facing + 180 + 2.5f);
                wave.setArcAttenuationWidth(5f);
                DistortionShader.addDistortion(wave);

                StandardLight light = new StandardLight();
                light.setType(1);
                light.setLocation(beamForm);
                light.setLocation2(beamTo);
                light.setColor(new Color(255, 255, 255, 100));
                light.setSize(25f);
                light.setIntensity(1f);
                light.fadeOut(0.3f);
                LightShader.addLight(light);
                times--;
            }
        }
    }

    public static class PiercedEntity {

        protected Vector2f shieldHit = null;
        protected Vector2f hullHit = null;
        protected Vector2f exit = null;
        protected CombatEntityAPI entity = null;
    }

    public static class SortByDistance implements Comparator<PiercedEntity> {

        private final Vector2f location;

        public SortByDistance(Vector2f location) {
            this.location = location;
        }

        @Override
        public int compare(PiercedEntity o1, PiercedEntity o2) {

            return Float.compare(
                    MathUtils.getDistanceSquared(o1.shieldHit == null ? o1.hullHit : o1.shieldHit, location),
                    MathUtils.getDistanceSquared(o2.shieldHit == null ? o2.hullHit : o2.shieldHit, location));
        }
    }
}
