package data.shipsystems.scripts;

import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.plugins.ShipSystemStatsScript;

public class NeutrinoSiegeModeStats implements ShipSystemStatsScript {

    // Credit to Deathfly for Fix when only armored weapon mounts is installed.
    public static final float DAMAGE_BONUS = -0.2F;
    public static final float WEAPON_RANGE_BONUS = 3F;//so the final weapon range should be 100% + 300%. Stack with skill and hull mod.
    public static final float TURN_PENALTY = 0.95F; //means the weapon turn rate will be 1-95%=5%. Stack with hull mod.

    public static final float BEAM_RANGE_PENALTY = 2F;//OK, beam is OPed on this one. try to make them only got 33.3% extra Range.
    public static final float BEAM_TURN_PENALTY = 0.5F;//And lease penalty.

    @Override
    public void apply(MutableShipStatsAPI stats, String id, State state, float effectLevel) {

        if (state == com.fs.starfarer.api.plugins.ShipSystemStatsScript.State.OUT) {
            stats.getMaxSpeed().unmodify(id);
            stats.getMaxTurnRate().unmodify(id);
        } else {
            stats.getMaxSpeed().modifyFlat(id, -200F);
            stats.getAcceleration().modifyPercent(id, 100F * effectLevel);
            stats.getDeceleration().modifyPercent(id, 100F * effectLevel);
            stats.getTurnAcceleration().modifyPercent(id, 0.0F * effectLevel);
            stats.getMaxTurnRate().modifyPercent(id, 0.0F);
        }
        float bonusPercent = 1F + DAMAGE_BONUS * effectLevel;
        stats.getEnergyWeaponDamageMult().modifyMult(id, bonusPercent);
        float weaponRangePercent = 1F + WEAPON_RANGE_BONUS * effectLevel;
        stats.getEnergyWeaponRangeBonus().modifyMult(id, weaponRangePercent);
        float turnMult = 1F - TURN_PENALTY * effectLevel;
        stats.getWeaponTurnRateBonus().modifyMult(id, turnMult);
        //neaf for beams.
        float beamRangePenalty = 1F / (1F + BEAM_RANGE_PENALTY * effectLevel);
        stats.getBeamWeaponRangeBonus().modifyMult(id, beamRangePenalty);
        float beamTurnMult = 1F - BEAM_TURN_PENALTY * effectLevel;
        stats.getBeamWeaponTurnRateBonus().modifyMult(id, beamTurnMult);
    }

    @Override
    public void unapply(MutableShipStatsAPI stats, String id) {
        stats.getEnergyWeaponDamageMult().unmodify(id);
        stats.getEnergyWeaponRangeBonus().unmodify(id);
        stats.getBeamWeaponRangeBonus().unmodify(id);
        stats.getWeaponTurnRateBonus().unmodify(id);
        stats.getBeamWeaponTurnRateBonus().unmodify(id);

        //stats.getArmorDamageTakenMult().unmodify(id);
        //stats.getHullDamageTakenMult().unmodify(id);
        //stats.getShieldDamageTakenMult().unmodify(id);
        //stats.getWeaponDamageTakenMult().unmodify(id);
        //stats.getEngineDamageTakenMult().unmodify(id);
        stats.getMaxSpeed().unmodify(id);
        stats.getMaxTurnRate().unmodify(id);
        stats.getTurnAcceleration().unmodify(id);
        stats.getAcceleration().unmodify(id);
        stats.getDeceleration().unmodify(id);

    }

    @Override
    public StatusData getStatusData(int index, State state, float effectLevel) {

        float dmgBonusPercent = -20F * effectLevel;
        if (index == 0) {
            return new StatusData((int) dmgBonusPercent + "% energy weapon damage", true);
        } else if (index == 1) {
            return new StatusData("engines disabled and turrets slowed", true);
        } else if (index == 2) {
            return new StatusData("+" + (int) effectLevel * 300 + "% energy projectile weapons range", false);
        } else if (index == 3) {
            return new StatusData("+" + (int) effectLevel * 33 + "% energy beam weapons range", false);
        }
        return null;
    }
}
