package org.lazywizard.lazylib.combat;

import com.fs.starfarer.api.combat.CombatEntityAPI;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;

public class AdvForce {

    /**
     * Apply force to an object. Remember Newton's Second Law.
     *
     * Force is multiplied by 100 to avoid requiring ridiculous force amounts.
     *
     * @param entity The {@link CombatEntityAPI} to apply the force to.
     * @param direction The directional vector of the force (this will
     * automatically be normalized).
     * @param forceBearingPoint Where the force should apply to.
     * @param force How much force to apply. Unit is how much it takes to modify
     * a 100 weight object's velocity by 1 su/sec.
     * <p>
     * @since 1.2
     */
    public static void applyForceEX(CombatEntityAPI entity, Vector2f forceBearingPoint, Vector2f direction, float force) {
        // Filter out forces without a direction
        if (direction.lengthSquared() == 0) {
            return;
        }
        // Force is far too weak otherwise
        force *= 100f;
        // Avoid divide-by-zero errors...
        float mass = Math.max(1f, entity.getMass());
        // Doing some vector calculate
        Vector2f BPtoMC = Vector2f.sub(entity.getLocation(), forceBearingPoint, null);
        Vector2f forceV = new Vector2f();
        direction.normalise(forceV);
        forceV.scale(force);
        // get moment vector
        Vector3f forceV3f = new Vector3f(forceV.x, forceV.y, 0);
        Vector3f torque = new Vector3f(BPtoMC.x, BPtoMC.y, 0);
        // get force vector
        BPtoMC.normalise(BPtoMC);
        // calculate acceleration
        BPtoMC.scale(Vector2f.dot(forceV, BPtoMC) / mass);
        // Apply velocity change
        Vector2f.add(BPtoMC, entity.getVelocity(), entity.getVelocity());

        // calculate moment change 
        Vector3f.cross(forceV3f, torque, torque);
        float angularAcc = torque.z / (0.5f * mass * entity.getCollisionRadius() * entity.getCollisionRadius());
        angularAcc = (float) Math.toDegrees(angularAcc);
        // Apply angular velocity change
        entity.setAngularVelocity(angularAcc + entity.getAngularVelocity());
    }
}
